%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  Purpose
%
%       Driver for the minimization of the Rosenbrock function. 
%
%       See Exercise 3.1 in J. Nocedal and S. J. Wright,
%       "Numerical Optimization", Springer-Verlag, 1999
%
%  Functions called: func.m, gradient.m, hessian.m
%
%
%  Written by L. N. Vicente.
%  February 21, 2000.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Begin.
%
% Clear work space and screen.
  clf;
  clear;
%
% Set formats.
  format compact;
  format long;
%
% Initializations.
%
% fid   file identifier for output file. No output if fid < 0.
%       Use fid = 1 for standard output (usually screen).
  fid = 1; 
%
% Parameters for line search.
  c1  = 10^(-4);
  rho = 0.5;
%
% Initial points.
%  x0 = [ 1.2 1.2 ]';
  x0 = [ -1.2 1 ]';
%
% Newton's method with a backtracking line search.
  iter  = 0; ifunc=0; ig=0; iH=0;
  x     = x0;
  [f]   = func(x);
  ifunc = ifunc+1;
  [g]   = gradient(x);
  ig    = ig+1;
  while ( norm(g) > sqrt(eps) )
          [H]       = hessian(x);
          iH        = iH+1;
          p         = - H\g;
          alpha     = 1;
          xalphap   = x + alpha*p;
          [falphap] = func(xalphap);
          ifunc     = ifunc+1;
          linearf   = f+c1*alpha*g'*p;
          while ( falphap > linearf )
%                 Backtrack.
                  alpha     = rho*alpha; 
                  xalphap   = x + alpha*p;
                  [falphap] = func(xalphap); 
                  ifunc     = ifunc+1;
                  linearf   = f+c1*alpha*g'*p;
          end
          x    = xalphap;
          f    = falphap;
          [g]  = gradient(x);
          ig   = ig+1;
          iter = iter + 1;
  end
%
  normg = norm(g);
  condH = cond(H);
% Printing.
  fprintf( fid,'\nNewton''s method with a backtracking line search \n');
  fprintf( fid,'\n|  iter  | fevals | gevals | Hevals |   norm(g)  |   cond(H)  | \n');
  format_str = '| %6d | %6d | %6d | %6d | %9.4e | %9.4e | \n \n';
  fprintf( fid, format_str, iter, ifunc, ig, iH, normg, condH);
%
% Newton's method.
  iter = 0; ifunc=0; ig=0; iH=0;
  x    = x0;    
  [g]  = gradient(x);
  ig   = ig+1;
  while ( norm(g) > sqrt(eps) )
          [H]  = hessian(x);
          iH   = iH+1;
          x    = x - H\g;
          [g]  = gradient(x);
          ig   = ig+1;
          iter = iter + 1;
  end
%
  normg = norm(g);
  condH = cond(H);
% Printing.
  fprintf( fid,'Newton''s method \n');
  fprintf( fid,'\n|  iter  | fevals | gevals | Hevals |   norm(g)  |   cond(H)  | \n');
  format_str = '| %6d | %6d | %6d | %6d | %9.4e | %9.4e | \n \n';
  fprintf( fid, format_str, iter, ifunc, ig, iH, normg, condH);
%
% Steepest descent method with a backtracking line search.
  iter  = 0; ifunc=0; ig=0; iH=0;
  x     = x0;
  [f]   = func(x);
  ifunc = ifunc+1;
  [g]   = gradient(x);
  ig    = ig+1;
  while ( norm(g) > sqrt(eps) )
          p         = -g;
          alpha     = 1;
          xalphap   = x + alpha*p;
          [falphap] = func(xalphap);
          ifunc     = ifunc+1;
          linearf   = f+c1*alpha*g'*p;
          while ( falphap > linearf )
%                 Backtrack.
                  alpha     = rho*alpha;
                  xalphap   = x + alpha*p;
                  [falphap] = func(xalphap);
                  ifunc     = ifunc+1;
                  linearf   = f+c1*alpha*g'*p;
          end
          x    = xalphap;
          f    = falphap;
          [g]  = gradient(x);
          ig   = ig+1;
          iter = iter + 1;
  end
%
  normg = norm(g);
  condH = cond(H);
% Printing.
  fprintf( fid, 'Steepest descent method with a backtracking line search \n');
  fprintf( fid,'\n|  iter  | fevals | gevals | Hevals |   norm(g)  |   cond(H)  | \n');
  format_str = '| %6d | %6d | %6d | %6d | %9.4e | %9.4e | \n \n';
  fprintf( fid, format_str, iter, ifunc, ig, iH, normg, condH);
%
% End of driver for the minimization of the Rosenbrock function.
